/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *      $RCSfile: CmdLabel.h,v $
 *      $Author: billh $        $Locker:  $                $State: Exp $
 *      $Revision: 1.2 $      $Date: 95/03/24 18:47:50 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * Command objects used to create, list, delete, or graph labels for measuring
 * geometries.
 *
 ***************************************************************************
 * REVISION HISTORY:
 *
 * $Log:	CmdLabel.h,v $
 * Revision 1.2  95/03/24  18:47:50  billh
 * Added copyright notice to top of file; made sure all virtual routines
 * are defined in the .C file, not in the .h file.
 * 
 * Revision 1.1  1995/03/04  05:33:08  billh
 * Initial revision
 *
 ***************************************************************************/
#ifndef CMDLABEL_H
#define CMDLABEL_H

#include "Command.h"
extern class Geometry;

// The following uses the Cmdtypes LABEL_ADD, LABEL_SHOW, LABEL_LIST,
// LABEL_GRAPH and LABEL_DELETE from the Command class


////////////////////// add a new label 
class CmdLabelAdd : public Command {
private:
  int cat;		// category for the label
  Geometry *geom;	// geometry object to add
  
protected:
  virtual int do_execute(void) ;

public:
  // constructor: category name, # items, item strings
  CmdLabelAdd(char *, int, char **, int UIid = (-1));
};


////////////////////// list the labels in a category
class CmdLabelList : public Command {
private:
  int cat;		// category to list

protected:
  virtual int do_execute(void) ;

public:
  // constructor: category name
  CmdLabelList(char *, int UIid = (-1));
};


////////////////////// toggle a geometry category on/off
class CmdLabelShow : public Command {
private:
  int cat;		// category to toggle
  int item;		// which item to toggle (if < 0, toggle entire category)
  int show;		// if T, turn on; if F, turn off

protected:
  virtual int do_execute(void) ;

public:
  // constructor: category name, show, item
  CmdLabelShow(char *, int, int, int UIid = (-1));
};


////////////////////// delete the Nth label in a category
class CmdLabelDelete : public Command {
private:
  int cat;		// category to delete from
  int item;		// which item to delete (if < 0 delete all in category)
protected:
  virtual int do_execute(void) ;

public:
  // constructor: category name, item
  CmdLabelDelete(char *, int, int UIid = (-1));
};


////////////////////// graph the values for the Nthe label in the given category
class CmdLabelGraph : public Command {
private:
  int cat;		// category to delete from
  int item;		// which item to graph
  char *graphCmd;	// command to execute to graph data; if null or empty,
  			// echo the data to the console
protected:
  virtual int do_execute(void) ;

public:
  // constructor: category name, item
  CmdLabelGraph(char *, int, char *, int UIid = (-1));
};


#endif
