/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *	$RCSfile: MoleculeFile.C,v $
 *	$Author: billh $	$Locker:  $		$State: Exp $
 *	$Revision: 1.6 $	$Date: 95/03/24 18:50:36 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * A MoleculeFile is inherited from Molecule, and supplies routines to 
 * create a molecule by reading data from a file.
 *
 ***************************************************************************
 * REVISION HISTORY:
 *
 * $Log:	MoleculeFile.C,v $
 * Revision 1.6  95/03/24  18:50:36  billh
 * Added copyright notice to top of file; made sure all virtual routines
 * are defined in the .C file, not in the .h file.
 * 
 * Revision 1.5  1995/02/12  07:21:11  dalke
 * moved PDB and PSF reads into seperate objects, making a "Molecule"
 * be much more general.
 *
 * Revision 1.4  1994/11/22  02:34:56  billh
 * Changed to not require StructureFile enum as type of argument to constructor.
 *
 * Revision 1.3  94/11/02  07:32:48  billh
 * Fixed bug preventing proper selection by resid.
 * 
 * Revision 1.2  1994/09/30  05:36:18  billh
 * Added externally-available string arrays for file type extensions.
 *
 * Revision 1.1  1994/09/17  09:11:36  billh
 * Initial revision
 *
 ***************************************************************************/
#ifdef ARCH_HPUX9
  static char ident[] = "@(#)$Header: /private/auto143000131/vmdsrc/vmd/billh/src/RCS/MoleculeFile.C,v 1.6 95/03/24 18:50:36 billh Exp $";
#endif

#include <math.h>
#include <string.h>
#include <ctype.h>
#include "MoleculeFile.h"
#include "Atom.h"
#include "utilities.h"


// string file extensions for the different known structure files
char *structureFileTypeNames[MoleculeFile::STRTYPES] = {
	"psf", "pdb", "edm", "graph", "raster3d"};


/////////////////////////////////////////////////////////////////////
// class constructor ... specifies the filename and either,
// a Scene or Displayable
MoleculeFile::MoleculeFile(char *fname, Scene *sc) : Molecule("File", sc) {
  do_construct(fname);
}

// this version uses a Displayable instead of Scene, so that it becomes a
// child Displayable to some parent.
MoleculeFile::MoleculeFile( char *fname, Displayable *dp) :
        Molecule("File", dp) {
  do_construct(fname);
}


///////////////////////// destructor
MoleculeFile::~MoleculeFile(void) {
  MSGDEBUG(1,"Deleting MoleculeFile ..." << sendmsg);

  delete [] strFile;
  delete [] strPath;
  delete [] strName;
}


// do main constructor tasks
void MoleculeFile::do_construct( char *fname) {
  strFile = stringdup(fname);
  breakup_filename(strFile, &strPath, &strName);
  if(!strPath)
    strPath = stringdup("");
  if(!strName)
    strName = stringdup("unknown");  
  MSGDEBUG(1,"Creating MoleculeFile, file='" << fname << "'");
}


// creates the molecule, using the info provided in the constructor.
// returns success, and also calls parent class create routines
int MoleculeFile::create(void) {

  // set the file name as the molecule's name
  set_name(strName);

  return Molecule::create();
}


// return the path of this molecule
char *MoleculeFile::file_path(void) {
  return strPath;
}


// return the name of this molecule
char *MoleculeFile::file_name(void) {
  return strName;
}
