/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *	$RCSfile: utilities.C,v $
 *	$Author: billh $	$Locker:  $		$State: Exp $
 *	$Revision: 1.11 $	$Date: 95/03/24 18:52:52 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * General utility routines and definitions.
 *
 ***************************************************************************
 * REVISION HISTORY:
 *
 * $Log:	utilities.C,v $
 * Revision 1.11  95/03/24  18:52:52  billh
 * Added copyright notice to top of file; made sure all virtual routines
 * are defined in the .C file, not in the .h file.
 * 
 * Revision 1.10  1995/03/04  05:06:34  billh
 * added midpoint and angle routines.
 *
 * Revision 1.9  94/11/02  05:19:23  dalke
 * Added str2bool - a string to boolean converter
 * 
 * Revision 1.8  1994/10/07  06:51:04  dalke
 * Corrected comment for cross_prod
 *
 * Revision 1.7  1994/09/28  17:04:42  billh
 * Changed fsqrt to sqrtf
 *
 * Revision 1.6  1994/09/23  21:48:07  dalke
 * made strupncmp a wrapper for strncasecmp
 *
 * Revision 1.5  1994/09/23  07:47:56  dalke
 * Rearranges the vector operations
 *
 * Revision 1.4  1994/09/16  21:34:14  dalke
 * added norm
 *
 * Revision 1.3  1994/09/16  21:26:44  dalke
 * added subtraction of vectors
 *
 * Revision 1.2  1994/09/03  11:06:29  dalke
 * added cross product and normalizing functions
 *
 * Revision 1.1  1994/08/24  03:10:37  billh
 * Initial revision
 *
 ***************************************************************************/
#ifdef ARCH_HPUX
  static char ident[] = "@(#)$Header: /private/auto143000131/vmdsrc/vmd/billh/src/RCS/utilities.C,v 1.11 95/03/24 18:52:52 billh Exp $";
#endif

#include <iostream.h>
#include <strings.h>
#include <ctype.h>
#include <sys/time.h>
#include <unistd.h>
#include <pwd.h>
#include <malloc.h>
#include <math.h>
#include "utilities.h"


// duplicate a string using c++ new call
char *stringdup(char *s) {
  char *rs;

  if(!s)
    return NULL;

  rs = new char[strlen(s) + 1];
  strcpy(rs,s);

  return rs;
}


// convert a string to upper case
char *stringtoupper(char *s) {
  register int i;

  if(s) {
    for(i=(strlen(s) - 1); i >= 0; i--)
      s[i] = toupper(s[i]);
  }

  return s;
}


// do upper-case comparison
int strupcmp(char *a, char *b) {
  char *ua, *ub;
  int retval;

  ua = stringtoupper(stringdup(a));
  ub = stringtoupper(stringdup(b));

  retval = strcmp(ua,ub);

  delete [] ub;
  delete [] ua;

  return retval;
}


// do upper-case comparison, up to n characters
int strupncmp(char *a, char *b, int n) {
  return strncasecmp(a, b, n);
}


// break a file name up into path + name, returning both in the specified
//	character pointers.  This creates storage for the new strings
//	by allocating space for them.
void breakup_filename(char *full, char **path, char **name) {
  char *namestrt;
  int pathlen;

  if(full == NULL) {
    *path = *name = NULL;
    return;
  } else if (strlen(full) == 0) {
    *path = new char[1];
    *name = new char[1];
    (*path)[0] = (*name)[0] = '\0';
    return;
  }

  // find start of final file name
  if((namestrt = strrchr(full,'/')) != NULL && strlen(namestrt) > 0) {
    namestrt++;
  } else {
    namestrt = full;
  }

  // make a copy of the name
  *name = stringdup(namestrt);

  // make a copy of the path
  pathlen = strlen(full) - strlen(*name);
  *path = new char[pathlen + 1];
  strncpy(*path,full,pathlen);
  (*path)[pathlen] = '\0';
} 


// break a configuration line up into tokens.  Returns tokenized string
// if the line is a valid one, i.e. the 2nd word is '=' and there is at 
// least one more word after the equal.
char *command_tokenize(char *newcmd, int *argc, char *argv[]) {
  char *cmd, *eqloc;

  // make a new string with a ' = ' instead of '='
  if(!(eqloc = strchr(newcmd,'=')))
    return NULL;
    
  cmd = new char[strlen(newcmd) + 3];
  strncpy(cmd,newcmd,(eqloc - newcmd));
  strcpy(cmd + (eqloc - newcmd)," = ");
  strcat(cmd,eqloc + 1);

  *argc = 0;

  argv[*argc] = strtok(cmd," ,;\t\n");
  if (argv[*argc] == NULL) {
    delete [] cmd;
    return NULL;
  }

  // see if the first token starts with '#'
  if(!strncmp(argv[0],"#",1)) {
    delete [] cmd;
    return NULL;
  }

  (*argc)++;

  // break up the rest of the string
  while ((argv[*argc] = strtok(NULL," ,;\t\n")) != NULL)
    (*argc)++;

  // make sure the 2nd word is "=", and there are 3 words or more
  if(*argc < 3 || strcmp(argv[1],"=")) {
    delete [] cmd;
    return NULL;
  }

  return cmd;
}


// break a configuration line up into tokens.
char *str_tokenize(char *newcmd, int *argc, char *argv[]) {
  char *cmd;

  cmd = stringdup(newcmd);
  *argc = 0;

  // initialize tokenizing calls
  argv[*argc] = strtok(cmd," ,;\t\n");

  // loop through words until end-of-string, or comment character, found
  while(argv[*argc] != NULL) {
    // see if the token starts with '#'
    if(argv[*argc][0] == '#') {
      delete [] cmd;
      return argv[0];
    } else {
      (*argc)++;		// another token in list
    }
    
    // scan for next token
    argv[*argc] = strtok(NULL," ,;\t\n");
  }

  return (*argc > 0 ? argv[0] : NULL);
}


// get the time of day from the system clock, and store it (in seconds)
double time_of_day(void) {
  struct timeval tm;
  struct timezone tz;

  gettimeofday(&tm, &tz);
  return((double)(tm.tv_sec) + (double)(tm.tv_usec)/1000000.0);
}


// return the username of the currently logged-on user
char *username(void) {
  struct passwd *pwentry;

  if((pwentry = getpwuid(getuid())) != NULL)
    return stringdup(pwentry->pw_name);
  else
    return NULL;
}


// return mem usage info (print it to stderr)
void meminfo(char *msg) {
  struct mallinfo mi;
  int i = 0;

  if(i) {
    mi = mallinfo();

    cerr << "Malloc info: ";
    if(msg)
      cerr << msg;
    cerr << "\n  Total space in arena: " << mi.arena
         << "\n  Total Ordinary blocks: " << mi.ordblks
         << "    Small blocks: " << mi.smblks
         << "\n  Space used: Ordinary = " << mi.uordblks
         << ", Small = " << mi.usmblks
         << "\n  Space free: Ordinary = " << mi.fordblks
         << ", Small = " << mi.fsmblks << endl;
  }
}

// take three 3-vectors and compute x2 cross x3; with the results
// in x1.  x1 must point to different memory than x2 or x3
// This returns a pointer to x1
float * cross_prod(float *x1, float *x2, float *x3)
{
  x1[0] =  x2[1]*x3[2] - x3[1]*x2[2];
  x1[1] = -x2[0]*x3[2] + x3[0]*x2[2];
  x1[2] =  x2[0]*x3[1] - x3[0]*x2[1];
  return x1;
}

// normalize a vector, and return a pointer to it
// Warning:  it changes the value of the vector!!
float * normalize(float *vect)
{
  float len = vect[0]*vect[0] + vect[1]*vect[1] + vect[2]*vect[2];
  if (len != 1) {
    len = sqrtf(len);
    vect[0] /= len;
    vect[1] /= len;
    vect[2] /= len;
  }
  return vect;
}


// find and return the norm of a 3-vector
float norm(float *vect)
{
  return sqrtf(vect[0]*vect[0] + vect[1]*vect[1] + vect[2]*vect[2]);
}


// compute the angle (in degrees) between two vectors a & b
extern float angle(float *a, float *b) {
  register float amag = sqrtf(a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
  register float bmag = sqrtf(b[0] * b[0] + b[1] * b[1] + b[2] * b[2]);
  register float dotprot = a[0] * b[0] + a[1] * b[1] + a[2] * b[2];

  return (57.2958 * acosf(dotprot / (amag * bmag)));
}



/// conversion from a string to a boolean
// returns TRUE == 1, FALSE == 0, and unknown == -1
extern int str2bool(const char *s)
{
  if (!strncasecmp("true", s, 5)) return 1;  // perhaps I went a bit overboard
  if (!strncasecmp("yes", s, 4)) return 1;
  if (!strncasecmp("yep", s, 4)) return 1;
  if (!strncasecmp("si", s, 3)) return 1;
  if (!strncasecmp("oui", s, 4)) return 1;
  if (!strncasecmp("1", s, 2)) return 1;
  if (!strncasecmp("on", s, 3)) return 1;
  if (!strncasecmp("yeah", s, 5)) return 1;
  if (!strncasecmp("ja", s, 3)) return 1;
  if (!strncasecmp("t", s, 1)) return 1;
  
  if (!strncasecmp("false", s, 6)) return 0; 
  if (!strncasecmp("no", s, 3)) return 0;
  if (!strncasecmp("nope", s, 5)) return 0;
  if (!strncasecmp("bo", s, 3)) return 0;
  if (!strncasecmp("non", s, 4)) return 0;
  if (!strncasecmp("0", s, 2)) return 0;
  if (!strncasecmp("off", s, 4)) return 0;
  if (!strncasecmp("nah", s, 4)) return 0;
  if (!strncasecmp("nine", s, 5)) return 0;
  if (!strncasecmp("f", s, 1)) return 0;
  
  return -1;
}
